@@ -629,6 +629,10 @@ class POP3RetrieverBase(RetrieverSkeleton):
             for (i, line) in enumerate(msglist):
                 try:
                     (msgnum, msgid) = line.split(None, 1)
+                    # Don't allow / in UIDs we store, as we look for that to 
+                    # detect old-style oldmail files.  Shouldn't occur in POP3
+                    # anyway.
+                    msgid = msgid.replace('/', '-')
                 except ValueError:
                     # Line didn't contain two tokens.  Server is broken.
                     raise getmailOperationError(
@@ -724,8 +728,8 @@ class POP3RetrieverBase(RetrieverSkeleton):
         # Handle password
         if self.conf.get('password', None) is None:
             self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'], 'pop3', 
-                self.log
+                self, self.conf['username'], self.conf['server'],
+                self.received_with, self.log
             )
         RetrieverSkeleton.initialize(self, options)
         try:
@@ -1015,7 +1019,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
 
         self.log.debug('selecting mailbox "%s"' % mailbox + os.linesep)
         try:
-            if self.app_options['delete'] or self.app_options['delete_after']:
+            if (self.app_options['delete'] or self.app_options['delete_after'] 
+                    or self.app_options['delete_bigger_than']):
                 read_only = False
             else:
                 read_only = True
@@ -1055,9 +1060,11 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 )
                 for line in response:
                     r = self._parse_imapattrresponse(line)
-                    msgid = (
-                        '%s/%s' % (self.uidvalidity, r['uid'])
-                    )
+                    # Don't allow / in UIDs we store, as we look for that to 
+                    # detect old-style oldmail files.  Can occur with IMAP, at 
+                    # least with some servers.
+                    uid = r['uid'].replace('/', '-')
+                    msgid = '%s/%s' % (self.uidvalidity, uid)
                     self._mboxuids[msgid] = r['uid']
                     self._mboxuidorder.append(msgid)
                     self.msgnum_by_msgid[msgid] = None
@@ -1144,11 +1151,62 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 # response[0] is None instead of a message tuple
                 raise getmailRetrievalError('failed to retrieve msgid %s' 
                                             % msgid)
+
+            # record mailbox retrieved from in a header
+            msg.add_header('X-getmail-retrieved-from-mailbox', 
+                           self.mailbox_selected)
+
+            # google extensions: apply labels, etc
+            if 'X-GM-EXT-1' in self.conn.capabilities:
+                metadata = self._getgmailmetadata(uid, msg)
+                for (header, value) in metadata.items():
+                    msg.add_header(header, value)
+
             return msg
 
         except imaplib.IMAP4.error, o:
             raise getmailOperationError('IMAP error (%s)' % o)
 
+    def _getgmailmetadata(self, uid, msg):
+        """
+        Add Gmail labels and other metadata which Google exposes through an
+        IMAP extension to headers in the message.
+        
+        See https://developers.google.com/google-apps/gmail/imap_extensions
+        """
+        try:
+            # ['976 (X-GM-THRID 1410134259107225671 X-GM-MSGID '
+            #   '1410134259107225671 X-GM-LABELS (labels space '
+            #   'separated) UID 167669)']
+            response = self._parse_imapuidcmdresponse('FETCH', uid,
+                '(X-GM-LABELS X-GM-THRID X-GM-MSGID)')
+        except imaplib.IMAP4.error, o:
+            self.log.warning('Could not fetch google imap extensions: %s' % o)
+            return {}
+
+        if not response:
+            return {}
+            
+        ext = re.search(
+            'X-GM-THRID (?P<THRID>\d+) X-GM-MSGID (?P<MSGID>\d+)'
+            ' X-GM-LABELS \((?P<LABELS>.*)\) UID',
+            response[0]
+        )
+        if not ext:
+            self.log.warning(
+                'Could not parse google imap extensions. Server said: %s'
+                % repr(response)
+            )
+            return ()
+
+        results = ext.groupdict()
+        metadata = {}
+        for item in ('LABELS', 'THRID', 'MSGID'):
+            if item in results and results[item]:
+                metadata['X-GMAIL-%s' % item] = results[item]
+
+        return metadata
+
     def _getmsgbyid(self, msgid):
         self.log.trace()
         if self.conf.get('use_peek', True):
@@ -1172,8 +1230,8 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         if (self.conf.get('password', None) is None
                 and not (HAVE_KERBEROS_GSS and self.conf['use_kerberos'])):
             self.conf['password'] = get_password(
-                self, self.conf['username'], self.conf['server'], 'imap',
-                self.log
+                self, self.conf['username'], self.conf['server'], 
+                self.received_with, self.log
             )
             
         RetrieverSkeleton.initialize(self, options)
